@@ -70,13 +70,19 @@ func (d *Destination) ToNativeDestination(option ...ToNativeOption) (*table.Dest
 	})
 	paths := make([]*table.Path, 0, len(d.Paths))
 	for _, p := range d.Paths {
-		path, err := p.ToNativePath(option...)
+		var path *table.Path
+		var err error
+		if p.Identifier > 0 {
+			path, err = p.ToNativePath()
+		} else {
+			path, err = p.ToNativePath(option...)
+		}
 		if err != nil {
 			return nil, err
 		}
 		paths = append(paths, path)
 	}
-	return table.NewDestination(nlri, paths...), nil
+	return table.NewDestination(nlri, 0, paths...), nil
 }
 
 func (p *Path) ToNativePath(option ...ToNativeOption) (*table.Path, error) {
@@ -113,8 +119,16 @@ func (p *Path) ToNativePath(option ...ToNativeOption) (*table.Path, error) {
 		pattr = append(pattr, p)
 	}
 	t := time.Unix(p.Age, 0)
+	nlri.SetPathIdentifier(p.Identifier)
+	nlri.SetPathLocalIdentifier(p.LocalIdentifier)
 	path := table.NewPath(info, nlri, p.IsWithdraw, pattr, t, false)
-	path.SetValidation(config.IntToRpkiValidationResultTypeMap[int(p.Validation)])
+	path.SetValidation(&table.Validation{
+		Status:          config.IntToRpkiValidationResultTypeMap[int(p.Validation)],
+		Reason:          table.IntToRpkiValidationReasonTypeMap[int(p.ValidationDetail.Reason)],
+		Matched:         NewROAListFromApiStructList(p.ValidationDetail.Matched),
+		UnmatchedAs:     NewROAListFromApiStructList(p.ValidationDetail.UnmatchedAs),
+		UnmatchedLength: NewROAListFromApiStructList(p.ValidationDetail.UnmatchedLength),
+	})
 	path.MarkStale(p.Stale)
 	path.SetUUID(p.Uuid)
 	if p.Filtered {
@@ -123,3 +137,22 @@ func (p *Path) ToNativePath(option ...ToNativeOption) (*table.Path, error) {
 	path.IsNexthopInvalid = p.IsNexthopInvalid
 	return path, nil
 }
+
+func NewROAListFromApiStructList(l []*Roa) []*table.ROA {
+	roas := make([]*table.ROA, 0, len(l))
+	for _, r := range l {
+		ip := net.ParseIP(r.Prefix)
+		rf := func(prefix string) bgp.RouteFamily {
+			a, _, _ := net.ParseCIDR(prefix)
+			if a.To4() != nil {
+				return bgp.RF_IPv4_UC
+			} else {
+				return bgp.RF_IPv6_UC
+			}
+		}(r.Prefix)
+		afi, _ := bgp.RouteFamilyToAfiSafi(rf)
+		roa := table.NewROA(int(afi), []byte(ip), uint8(r.Prefixlen), uint8(r.Maxlen), r.As, net.JoinHostPort(r.Conf.Address, r.Conf.RemotePort))
+		roas = append(roas, roa)
+	}
+	return roas
+}
