@@ -52,6 +52,21 @@ env.abort_exception = RuntimeError
 output.stderr = False
 
 
+def community_str(i):
+    """
+    Converts integer in to colon separated two bytes decimal strings like
+    BGP Community or Large Community representation.
+
+    For example, this function converts 13107300 = ((200 << 16) | 100)
+    into "200:100".
+    """
+    values = []
+    while i > 0:
+        values.append(str(i & 0xffff))
+        i >>= 16
+    return ':'.join(reversed(values))
+
+
 def wait_for_completion(f, timeout=120):
     interval = 1
     count = 0
@@ -311,7 +326,7 @@ class BGPContainer(Container):
                  flowspec=False, bridge='', reload_config=True, as2=False,
                  graceful_restart=None, local_as=None, prefix_limit=None,
                  v6=False, llgr=None, vrf='', interface='', allow_as_in=0,
-                 remove_private_as=None, replace_peer_as=False):
+                 remove_private_as=None, replace_peer_as=False, addpath=False):
         neigh_addr = ''
         local_addr = ''
         it = itertools.product(self.ip_addrs, peer.ip_addrs)
@@ -355,7 +370,8 @@ class BGPContainer(Container):
                             'vrf': vrf,
                             'allow_as_in': allow_as_in,
                             'remove_private_as': remove_private_as,
-                            'replace_peer_as': replace_peer_as}
+                            'replace_peer_as': replace_peer_as,
+                            'addpath': addpath}
         if self.is_running and reload_config:
             self.create_config()
             self.reload_config()
@@ -378,18 +394,31 @@ class BGPContainer(Container):
     def add_route(self, route, rf='ipv4', attribute=None, aspath=None,
                   community=None, med=None, extendedcommunity=None,
                   nexthop=None, matchs=None, thens=None,
-                  local_pref=None, reload_config=True):
-        self.routes[route] = {'prefix': route,
-                              'rf': rf,
-                              'attr': attribute,
-                              'next-hop': nexthop,
-                              'as-path': aspath,
-                              'community': community,
-                              'med': med,
-                              'local-pref': local_pref,
-                              'extended-community': extendedcommunity,
-                              'matchs': matchs,
-                              'thens': thens}
+                  local_pref=None, identifier=None, reload_config=True):
+        if route not in self.routes:
+            self.routes[route] = []
+        self.routes[route].append({
+            'prefix': route,
+            'rf': rf,
+            'attr': attribute,
+            'next-hop': nexthop,
+            'as-path': aspath,
+            'community': community,
+            'med': med,
+            'local-pref': local_pref,
+            'extended-community': extendedcommunity,
+            'identifier': identifier,
+            'matchs': matchs,
+            'thens': thens,
+        })
+        if self.is_running and reload_config:
+            self.create_config()
+            self.reload_config()
+
+    def del_route(self, route, identifier=None, reload_config=True):
+        if route not in self.routes:
+            return
+        self.routes[route] = [p for p in self.routes[route] if p['identifier'] != identifier]
         if self.is_running and reload_config:
             self.create_config()
             self.reload_config()
